// src/stores/offline.ts (Example modification)
import { defineStore } from 'pinia';
import { ref, computed } from 'vue';
// import { LocalStorage } from 'quasar'; // REMOVE
import { useStorage } from '@vueuse/core'; // VueUse alternative
import { useNotificationStore } from '@/stores/notifications'; // Your custom notification store

// ... (interfaces remain the same)

export const useOfflineStore = defineStore('offline', () => {
    // const $q = useQuasar(); // REMOVE
    const notificationStore = useNotificationStore();
    const isOnline = ref(navigator.onLine);
    interface OfflineAction {
        id: string;
        timestamp: number;
        type: string;
        // Add other necessary fields
    }

    interface ConflictData {
        localVersion: unknown; // Replace with proper type
        serverVersion: unknown; // Replace with proper type
        action: OfflineAction;
    }

    // Use useStorage for reactive localStorage
    const pendingActions = useStorage<OfflineAction[]>('offline-actions', []);
    
    const isProcessingQueue = ref(false);
    const showConflictDialog = ref(false); // You'll need to implement this dialog
    const currentConflict = ref<ConflictData | null>(null);

    // init is now handled by useStorage automatically loading the value

    // saveToStorage is also handled by useStorage automatically saving on change

    const addAction = (action: Omit<OfflineAction, 'id' | 'timestamp'>) => {
        const newAction: OfflineAction = {
            ...action,
            id: crypto.randomUUID(),
            timestamp: Date.now(),
        };
        pendingActions.value.push(newAction);
        // useStorage handles saving
    };

    const processQueue = async () => {
        if (isProcessingQueue.value || !isOnline.value) return;
        isProcessingQueue.value = true;
        const actionsToProcess = [...pendingActions.value]; // Create a copy to iterate

        for (const action of actionsToProcess) {
            try {
                await processAction(action); // processAction needs to use your actual API client
                pendingActions.value = pendingActions.value.filter(a => a.id !== action.id);
            } catch (error) {
                if (error instanceof Error && 'response' in error && typeof error.response === 'object' && error.response && 'status' in error.response && error.response.status === 409) {
                    notificationStore.addNotification({
                        type: 'warning',
                        message: 'Item was modified by someone else. Please review.',
                        // actions: [ ... ] // Custom actions for notifications would be more complex
                    });
                    // Here you would trigger the conflict resolution dialog
                    // For example, find the item and its server version, then:
                    // currentConflict.value = { localVersion: ..., serverVersion: ..., action };
                    // showConflictDialog.value = true;
                    // The loop should probably pause or handle this conflict before continuing
                    console.warn('Conflict detected for action:', action.id, error);
                    // Break or decide how to handle queue processing on conflict
                    break; 
                } else {
                    console.error('Failed to process offline action:', action.id, error);
                    notificationStore.addNotification({
                        type: 'error',
                        message: `Failed to sync action: ${action.type}`,
                    });
                }
            }
        }
        isProcessingQueue.value = false;
    };
    
    // processAction needs to be implemented with your actual API calls
    const processAction = async (action: OfflineAction) => {
      console.log('Processing action (TODO: Implement API call):', action);
      // Example:
      // import { apiClient } from '@/services/api';
      // import { API_ENDPOINTS } from '@/config/api-config';
      // switch (action.type) {
      //   case 'add':
      //     // Assuming action.data is { listId: string, itemData: { name: string, quantity?: string } }
      //     // await apiClient.post(API_ENDPOINTS.LISTS.ITEMS(action.data.listId), action.data.itemData);
      //     break;
      //   // ... other cases
      // }
      // Simulate async work
      return new Promise(resolve => setTimeout(resolve, 500));
    };


    const setupNetworkListeners = () => {
      window.addEventListener('online', () => {
        isOnline.value = true;
        processQueue().catch(err => console.error("Error processing queue on online event:", err));
      });
      window.addEventListener('offline', () => {
        isOnline.value = false;
      });
    };
    
    setupNetworkListeners(); // Call this once

    const hasPendingActions = computed(() => pendingActions.value.length > 0);
    const pendingActionCount = computed(() => pendingActions.value.length);

    const handleConflictResolution = (resolution: { version: 'local' | 'server' | 'merge'; action: OfflineAction; mergedData?: Record<string, unknown> }) => {
        console.log('Conflict resolution chosen:', resolution);
        // TODO: Implement logic to apply the chosen resolution
        // This might involve making another API call with the resolved data
        // or updating local state and then trying to sync again.
        // After resolving, remove the action from pending or mark as resolved.
        // For now, just remove it as an example:
        pendingActions.value = pendingActions.value.filter(a => a.id !== resolution.action.id);
        showConflictDialog.value = false;
        currentConflict.value = null;
        processQueue().catch(err => console.error("Error processing queue after conflict resolution:", err)); // Try processing queue again
    };

    return {
        isOnline,
        pendingActions,
        hasPendingActions,
        pendingActionCount,
        showConflictDialog,
        currentConflict,
        addAction,
        processQueue,
        handleConflictResolution,
    };
});